/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             DeltaLog management
*  SCOPE :              DeltaLog management
*  FILE :               delta.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#ifndef __delta_HXX
#include "delta.hxx"
#endif

/*****************************************************************************/

LinkedList Delta::_deltalist ;


/*= EXTERNAL FUNCTION =======================================================*/
DeltaVue* Delta::createVue()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  DeltaVue* dv = new DeltaVue ;
  Delta* n = (Delta*)_deltalist.head() ;
  while ( (n != NULL) && (n->islog()) && (n->isactive()) ) {
    n = (Delta*)n->next() ;
  }
  if (n != NULL) {
    dv->insertBefore(n) ;
  } else {
    _deltalist.insertTail(dv) ;
  }
  return dv ;
}


/*= EXTERNAL FUNCTION =======================================================*/
DeltaLog* Delta::createLog(char* name)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  DeltaLog* dl = new DeltaLog(name) ;
  _deltalist.insertHead(dl) ;
  return dl ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int Delta::read(Page* p, PID pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Delta* d = (Delta*)this->prev() ;
  if (d != NULL) {
    return d->read(p, pid) ;
  } else {
    return -1 ;
  } 
}


/*= EXTERNAL FUNCTION =======================================================*/
int Delta::delta(PID pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Delta* d = (Delta*)this->prev() ;
  if (d != NULL) {
    return d->delta(pid) ;
  } else {
    return FALSE ;
  } 
}


/*= EXTERNAL FUNCTION =======================================================*/
DeltaVue::~DeltaVue()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  Delta* d = (Delta*)this->prev() ;
  this->remove() ;    
  if (this->next() == NULL) {
    if ( (d != NULL) && (d->islog()) && (d->isactive() == FALSE) ) {
      ((DeltaLog*)d)->close() ;
    }
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
int DeltaVue::islog()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  return FALSE ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int DeltaVue::isactive()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  return FALSE ;
}


/*= EXTERNAL FUNCTION =======================================================*/
DeltaLog::~DeltaLog()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  IterPageSet iter(_logindex) ;
  for (; iter.more(); iter.next()) {
    delete iter.get() ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
int DeltaLog::read(Page* p, PID pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  PageHdlLog* pg = (PageHdlLog*)_logindex[pid] ;
  if (pg != NULL) {
    this->Log::read(p, pg->numpage()) ;
    return 0 ;
  } else {    
    return this->Delta::read(p, pid) ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
int DeltaLog::write(Page* p, PID pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  int pgnum = this->Log::write(p, pid) ;
  PageHdlLog* pg = new PageHdlLog(pgnum, pid) ;
  this->_logindex += pg ;
  return pgnum ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int DeltaLog::delta(PID pid)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  PageHdlLog* pg = (PageHdlLog*)_logindex[pid] ;
  if (pg != NULL) {    
    return TRUE ;
  } else {    
    return this->Delta::delta(pid) ;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void DeltaLog::close()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  *this->_inuse = FALSE ;
  this->sync() ;
  if (this->next() != NULL) return ;
  ::close(this->_fd) ;
  Delta* d = (Delta*)this->prev() ;
  this->remove() ;
  delete this ;
  while ((d != NULL) && (d->islog())) {
    Delta* d2 = d ;
    d = (Delta*)d->prev() ;
    if (d2->isactive() == FALSE) {
      d2->remove() ;
      delete d2 ;
    }
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
int DeltaLog::islog()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  return TRUE ;
}


/*= EXTERNAL FUNCTION =======================================================*/
int DeltaLog::isactive()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  return this->Log::isactive() ;
}

